// -*- c++ -*-

// Use a DIMACS max flow file as input.
// sub_graph_adaptor_demo < dimacs_max_flow_file
// This program computes a maximum number of edge-disjoint shortest paths
// between s and t.

#include <iostream>
#include <fstream>

#include <lemon/smart_graph.h>
#include <lemon/dijkstra.h>
#include <lemon/maps.h>
#include <lemon/graph_adaptor.h>
#include <lemon/dimacs.h>
#include <lemon/preflow.h>
#include <tight_edge_filter_map.h>

using namespace lemon;

using std::cout;
using std::endl;

int main(int argc, char *argv[]) 
{
  if(argc<2)
  {
      std::cerr << "USAGE: sub_graph_adaptor_demo <input_file.dim>" << std::endl;
      std::cerr << "The file 'input_file.dim' has to contain a max flow instance in DIMACS format (e.g. sub_graph_adaptor_demo.dim is such a file)." << std::endl;
      return 0;
  }


  //input stream to read the graph from
  std::ifstream is(argv[1]);

  typedef SmartGraph Graph;

  typedef Graph::Edge Edge;
  typedef Graph::Node Node;
  typedef Graph::EdgeIt EdgeIt;
  typedef Graph::NodeIt NodeIt;
  typedef Graph::EdgeMap<int> LengthMap;

  Graph g;
  Node s, t;
  LengthMap length(g);

  readDimacs(is, g, length, s, t);

  cout << "edges with lengths (of form id, source--length->target): " << endl;
  for(EdgeIt e(g); e!=INVALID; ++e) 
    cout << " " << g.id(e) << ", " << g.id(g.source(e)) << "--" 
	 << length[e] << "->" << g.id(g.target(e)) << endl;

  cout << "s: " << g.id(s) << " t: " << g.id(t) << endl;

  typedef Dijkstra<Graph, LengthMap> Dijkstra;
  Dijkstra dijkstra(g, length);
  dijkstra.run(s);

  // This map returns true exactly for those edges which are 
  // tight w.r.t the length funcion and the potential 
  // given by the dijkstra algorithm.
  typedef TightEdgeFilterMap<Graph, const Dijkstra::DistMap, LengthMap> 
    TightEdgeFilter;
  TightEdgeFilter tight_edge_filter(g, dijkstra.distMap(), length);

//  ConstMap<Node, bool> const_true_map(true);
  // This graph contains exaclty the tight edges.
// typedef SubGraphAdaptor<Graph, ConstMap<Node, bool>, TightEdgeFilter> SubGW;
  typedef EdgeSubGraphAdaptor<Graph, TightEdgeFilter> SubGW;
  SubGW gw(g, tight_edge_filter);

  ConstMap<Edge, int> const_1_map(1);
  Graph::EdgeMap<int> flow(g, 0);
  // Max flow between s and t in the graph of tight edges.
  Preflow<SubGW, int, ConstMap<Edge, int>, Graph::EdgeMap<int> > 
    preflow(gw, s, t, const_1_map, flow);
  preflow.run();

  cout << "maximum number of edge-disjoint shortest paths: " 
       << preflow.flowValue() << endl;
  cout << "edges of the maximum number of edge-disjoint shortest s-t paths: " 
       << endl;
  for(EdgeIt e(g); e!=INVALID; ++e) 
    if (flow[e])
      cout << " " << g.id(e) << ", "
	   << g.id(g.source(e)) << "--" 
	   << length[e] << "->" << g.id(g.target(e)) << endl;
}
